@@ -18,7 +18,7 @@ import getpass
 import time
 import xml.etree.ElementTree as ET
 from collections import namedtuple
-from datetime import datetime
+from datetime import datetime, timedelta
 from pprint import pformat
 
 import ibm_boto3
@@ -27,9 +27,9 @@ import requests
 from ibm_botocore.client import Config
 
 try:
-    from utilities import IBMCloudAccess
+    from utilities import IBMCloudAccess, confirm_action
 except Exception:
-    from .utilities import IBMCloudAccess
+    from .utilities import IBMCloudAccess, confirm_action
 from requests.exceptions import HTTPError
 import json
 import logging
@@ -216,7 +216,9 @@ class ParsedUrl(object):
             "mil": "s3.mil.eu.cloud-object-storage.appdomain.cloud",
             "us-south": "s3.us-south.cloud-object-storage.appdomain.cloud",
             "us-east": "s3.us-east.cloud-object-storage.appdomain.cloud",
+            "ca-tor": "s3.ca-tor.cloud-object-storage.appdomain.cloud",
             "jp-tok": "s3.jp-tok.cloud-object-storage.appdomain.cloud",
+            "jp-osa": "s3.jp-osa.cloud-object-storage.appdomain.cloud",
             "ap-geo": "s3.ap.cloud-object-storage.appdomain.cloud",
             "ap": "s3.ap.cloud-object-storage.appdomain.cloud",
             "tok-ap-geo": "s3.tok.ap.cloud-object-storage.appdomain.cloud",
@@ -251,7 +253,9 @@ class ParsedUrl(object):
             "s3.eu-gb.cloud-object-storage.appdomain.cloud",
             "s3.eu-de.cloud-object-storage.appdomain.cloud",
             "s3.au-syd.cloud-object-storage.appdomain.cloud",
+            "s3.ca-tor.cloud-object-storage.appdomain.cloud",
             "s3.jp-tok.cloud-object-storage.appdomain.cloud",
+            "s3.jp-osa.cloud-object-storage.appdomain.cloud",
             "s3.us-west.cloud-object-storage.test.appdomain.cloud",
             # cross-region
             "s3.us.cloud-object-storage.appdomain.cloud",
@@ -313,6 +317,8 @@ class ParsedUrl(object):
         cos_url : str
             COS URL
         """
+        if (cos_url[-1] != "/") & (cos_url.count("/") < 4):
+            cos_url += "/"
         nt = namedtuple("COSURL", "endpoint bucket prefix")
         mycontainer = nt(
             self.get_endpoint(cos_url),
@@ -459,7 +465,73 @@ class COSClient(ParsedUrl, IBMCloudAccess):
 
         return _get_default_s3_client(endpoint)
 
-    def list_cos_objects(self, cos_url, size_unit=None):
+    def copy_daily_objects(self, source_cos_url, target_cos_url, buffer_days=1):
+        """
+        Copy all objects from a source location in a per day folder on a target location, with a sliding window buffer of days.
+
+        Parameters
+        ------------
+        source_cos_url: str
+            Your source data in format cos://us-south/<bucket-name>/object_path/
+
+        source_cos_url: str
+            Your target data in format cos://us-south/<bucket-name>/object_path/
+
+        buffer_days: sint
+            Number of additional days before and after each day for which to copy the landed objects into the single target day folder.
+
+        Returns
+        ----------
+        None
+
+        Raises
+        -------
+        ValueError
+            if COS URL is invalid
+        """
+
+        if not self.is_valid_cos_url(source_cos_url):
+            msg = "Not a valid COS URL: {}".format(source_cos_url)
+            raise ValueError(msg)
+        if not self.is_valid_cos_url(target_cos_url):
+            msg = "Not a valid COS URL: {}".format(target_cos_url)
+            raise ValueError(msg)
+
+        source_cos_url = self.get_exact_url(source_cos_url)
+        source_url_parsed = self.analyze_cos_url(source_cos_url)
+        target_cos_url = self.get_exact_url(target_cos_url)
+        target_url_parsed = self.analyze_cos_url(target_cos_url)
+        target_prefix=target_url_parsed.prefix
+        if len(target_prefix) > 0:
+            if target_prefix[-1] != '/':
+                target_prefix = target_prefix + "/"
+        cos_client = self._get_cos_client(target_url_parsed.endpoint)
+
+        source_objects = self.list_cos_objects(source_cos_url)
+        source_objects["LastModified"] = source_objects["LastModified"].dt.date
+        start_date = source_objects.LastModified.min()
+        end_date = source_objects.LastModified.max()
+        for n in range(int((end_date - start_date).days) + 1):
+            current_date = start_date + timedelta(n-1)
+            current_objects = source_objects[(source_objects['LastModified'] >= current_date - timedelta(days=buffer_days)) &
+                                             (source_objects['LastModified'] <= current_date + timedelta(days=buffer_days))]
+
+            for source_index, source_object in current_objects.iterrows():
+                current_source = source_url_parsed.bucket + "/" + source_object['Object']
+                if "/_schema_as_json" in current_source or "/_checkpoint/" in current_source:
+                    continue # Don't copy SQL Query stream data landing checkpoint and schema objects.
+                current_target_object = source_object['Object'][len(source_url_parsed.prefix):]
+                current_target_prefix = "{}_date_landed={}".format(target_prefix, current_date)
+                cos_client.copy_object(
+                    Bucket=target_url_parsed.bucket,
+                    CopySource=current_source,
+                    Key=current_target_prefix + "/" + current_target_object
+                )
+            if len(current_objects)>0:
+                cos_client.put_object(Body='', Bucket=target_url_parsed.bucket, Key=current_target_prefix + "/_SUCCESS")
+                print("Copied {} objects to bucket {} into folder {}.".format(len(current_objects), target_url_parsed.bucket, current_target_prefix))
+
+    def list_cos_objects(self, cos_url, size_unit=None, sort_by_size=False):
         """
         List all objects in the current COS URL. Also, please read the note to see the role of trailing slash in the COS URL.
 
@@ -533,6 +605,8 @@ class COSClient(ParsedUrl, IBMCloudAccess):
             result.Size = result.Size / mapping[size_unit]
         else:
             result = pd.DataFrame()
+        if sort_by_size is True:
+            result.sort_values("Size", inplace=True, ascending=False)
         return result
 
     def delete_empty_objects(self, cos_url):
@@ -556,11 +630,57 @@ class COSClient(ParsedUrl, IBMCloudAccess):
             if int(result_objects.Size[row]) == 0:
                 cos_client.delete_object(Bucket=bucket, Key=result_objects.Object[row])
 
-    def delete_objects(self, cos_url, dry_run=False):
+    def _get_objects(self, http_string, result_location):
+        """
+        return upt-to-1000 objects [as constrained by
+        https://ibm.github.io/ibm-cos-sdk-python/reference/services/s3.html]
+
+        Returns:
+        -------
+        list of dict
+            The element in the list is a dict which contains a single key "Key",
+            with value as the suffix (after the bucketname until the end of COS URL)
+
+            Return empty list if there is no object or can't read the COS URL
+
+        """
+        response = requests.get(http_string, headers=self.request_headers,)
+
+        bucket_objects = []
+        if response.status_code == 200 or response.status_code == 201:
+            ns = {"s3": "http://s3.amazonaws.com/doc/2006-03-01/"}
+            responseBodyXMLroot = ET.fromstring(response.content)
+            bucket_name = responseBodyXMLroot.find("s3:Name", ns).text
+            bucket_objects = []
+            if responseBodyXMLroot.findall("s3:Contents", ns):
+                for contents in responseBodyXMLroot.findall("s3:Contents", ns):
+                    key = contents.find("s3:Key", ns)
+                    bucket_objects.append({"Key": key.text})
+        else:
+            print(
+                "Result object listing for COS URL at {} failed with http code {}".format(
+                    result_location, response.status_code
+                )
+            )
+        return bucket_objects
+
+    def delete_objects(self, cos_url, dry_run=False, confirm=False, get_result=True):
         """delete all objects stored at the given COS URL
 
         https://<cos-url>/<bucket>?prefix=<prefix-path>
 
+        Parameters
+        ----------
+        confirm: bool, default=False
+            confirm before deleting
+        get_result: bool, default=True
+            return the result, can be slow on large number of objects
+
+        Returns
+        -------
+        pd.DataFrame
+            A single column dataframe: ["Deleted Object"]
+
         Notes
         ------
 
@@ -622,49 +742,50 @@ class COSClient(ParsedUrl, IBMCloudAccess):
             + "?prefix="
             + result_location[fourth_slash + 1 :]
         )
-        response = requests.get(http_string, headers=self.request_headers,)
         if dry_run is True:
             import pprint
 
             print("http request {}".format(http_string))
             pprint.pprint(" response = {}".format(response.content))
+            deleted_list_df = pd.DataFrame(columns=["Deleted Object"])
+            return deleted_list_df
 
-        if response.status_code == 200 or response.status_code == 201:
-            ns = {"s3": "http://s3.amazonaws.com/doc/2006-03-01/"}
-            responseBodyXMLroot = ET.fromstring(response.content)
-            bucket_name = responseBodyXMLroot.find("s3:Name", ns).text
-            bucket_objects = []
-            if responseBodyXMLroot.findall("s3:Contents", ns):
-                for contents in responseBodyXMLroot.findall("s3:Contents", ns):
-                    key = contents.find("s3:Key", ns)
-                    bucket_objects.append({"Key": key.text})
-            else:
-                print(
-                    "There are no result objects for the COS URL {url}".format(
-                        url=cos_url
+        answer = True
+        if confirm is True:
+            answer = confirm_action("delete {}".format(cos_url))
+
+        deleted_list_df = pd.DataFrame(columns=["Deleted Object"])
+        if answer is True:
+            cos_client = self._get_cos_client(url_parsed.endpoint)
+            bucket_name = self.get_bucket(cos_url)
+            first_check = True
+            while True:
+                bucket_objects = self._get_objects(http_string, result_location)
+                if len(bucket_objects) > 0:
+                    first_check = False
+                    deleted_list_df = self._delete_objects(
+                        cos_client, bucket_name, bucket_objects, deleted_list_df
                     )
-                )
-                return
-        else:
-            print(
-                "Result object listing for COS URL at {} failed with http code {}".format(
-                    result_location, response.status_code
-                )
-            )
-            return
+                else:
+                    if first_check is True:
+                        print(
+                            "There are no result objects for the COS URL {url}".format(
+                                url=cos_url
+                            )
+                        )
+                    break
+        return deleted_list_df
 
-        cos_client = self._get_cos_client(url_parsed.endpoint)
+    def _delete_objects(
+        self, cos_client, bucket_name, bucket_objects, deleted_list_df=None
+    ):
+        """ delete up-to 1000 objects
+        """
+        response = cos_client.delete_objects(
+            Bucket=bucket_name, Delete={"Objects": bucket_objects}
+        )
 
-        if dry_run is True:
-            print(type(cos_client))
-            print(cos_client)
-            deleted_list_df = pd.DataFrame(columns=["Deleted Object"])
-        else:
-            response = cos_client.delete_objects(
-                Bucket=bucket_name, Delete={"Objects": bucket_objects}
-            )
-
-            deleted_list_df = pd.DataFrame(columns=["Deleted Object"])
+        if deleted_list_df is not None:
             for deleted_object in response["Deleted"]:
                 deleted_list_df = deleted_list_df.append(
                     [{"Deleted Object": deleted_object["Key"]}],
@@ -862,6 +983,8 @@ class COSClient(ParsedUrl, IBMCloudAccess):
         """
 
         def sizeof_fmt(num, suffix="B"):
+            if num is None:
+                return None
             for unit in ["", "K", "M", "G", "T", "P", "E", "Z"]:
                 if abs(num) < 1024.0:
                     return "%3.1f %s%s" % (num, unit, suffix)
